'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _unirest = require('unirest');

var _unirest2 = _interopRequireDefault(_unirest);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var remove = function remove(url, json, opts) {
  return send(_extends({ req: _unirest2.default.delete(), url: url, json: json }, opts));
};

var download = function download(url, serverPath, opts) {
  return send(_extends({ req: _unirest2.default.get(), url: url, serverPath: serverPath }, opts));
};

var get = function get(url, opts) {
  return send(_extends({ req: _unirest2.default.get(), url: url }, opts));
};

var patchJson = function patchJson(url, json, opts) {
  return send(_extends({ req: _unirest2.default.patch(), url: url, json: json }, opts));
};

var post = function post(url, payload, opts) {
  return send(_extends({ req: _unirest2.default.post(), url: url, payload: payload }, opts));
};

var postJson = function postJson(url, json, opts) {
  return send(_extends({ req: _unirest2.default.post(), url: url, json: json }, opts));
};

var upload = function upload(url, file, opts) {
  return send(_extends({ req: _unirest2.default.post(), url: url, file: file }, opts));
};

var send = function send() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      req = _ref.req,
      url = _ref.url,
      timeout = _ref.timeout,
      _ref$throwErrors = _ref.throwErrors,
      throwErrors = _ref$throwErrors === undefined ? true : _ref$throwErrors,
      serverPath = _ref.serverPath,
      payload = _ref.payload,
      json = _ref.json,
      file = _ref.file,
      headers = _ref.headers,
      formData = _ref.formData,
      completeResponse = _ref.completeResponse;

  return new Promise(function (resolve, reject) {
    req.url(url);
    if (timeout) req.timeout(timeout);
    if (headers) req.headers(headers);

    if (serverPath) {
      req.encoding('binary');
    } else if (payload) {
      req.send(payload);
    } else if (json) {
      req.type('json').send(json);
    } else if (file) {
      req.attach('file', file);
    } else if (formData) {
      for (var field in formData) {
        req.field(field, formData[field]);
      }
    }
    req.end(function (res) {
      if (res.ok) {
        if (serverPath) {
          _fs2.default.writeFileSync(serverPath, completeResponse ? res : res.body, { encoding: 'binary' });
          resolve(serverPath);
        } else {
          if (completeResponse) {
            resolve(res);
          } else {
            resolve(res.body);
          }
        }
      } else {
        if (res.error && !res.body) {
          res.body = typeof res.error === 'string' ? { Message: res.error } : res.error;
        } else if (typeof res.body === 'string') {
          res.body = { Message: res.body };
        }
        res.body.RequestUrl = req.options.url;
        res.body.RequestMethod = req.options.method;
        if (throwErrors) return reject(res.body);
        if (completeResponse) {
          resolve(res);
        } else {
          resolve(res.body);
        }
      }
    });
  });
};

exports.default = { remove: remove, download: download, get: get, patchJson: patchJson, post: post, postJson: postJson, upload: upload };